package com.unitech.test.sttdemo;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.speech.RecognitionListener;
import android.speech.RecognizerIntent;
import android.speech.SpeechRecognizer;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.concurrent.atomic.AtomicBoolean;


public class MainActivity extends AppCompatActivity {
    private static final int REQ_PERM = 1001;

    private SpeechRecognizer speechRecognizer;
    private SpeechRecognizerListener speechRecognizerListener;
    private TextView textView;
    private Button btn;

    private AtomicBoolean isInProgress = new AtomicBoolean(false);

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        textView = findViewById(R.id.textView);
        btn = findViewById(R.id.button);
        btn.setOnClickListener(this::onClick);

        if (SpeechRecognizer.isRecognitionAvailable(this)) {
            speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this);
            speechRecognizerListener = new SpeechRecognizerListener();
            speechRecognizer.setRecognitionListener(speechRecognizerListener);
        } else {
            textView.setText(R.string.recog_unavailable);
            btn.setVisibility(View.GONE);
        }
    }

    private void onClick(View view){
        if (isInProgress.get()) {
            stopSTT();
        } else {
            checkPermissions();
        }
    }

    private void checkPermissions(){
        if (checkSelfPermission(Manifest.permission.RECORD_AUDIO)
                == PackageManager.PERMISSION_GRANTED) {
            startSTT();
        } else {
            requestPermissions(new String[]{Manifest.permission.RECORD_AUDIO}, REQ_PERM);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == REQ_PERM) {
            if (grantResults.length > 0) {
                if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    startSTT();
                } else {
                    Toast.makeText(this, "Audio recording permissions denied.", Toast.LENGTH_LONG).show();
                }
            }
        }
    }

    private void startSTT(){
        if (!isInProgress.compareAndSet(false, true)) {
            return;
        }

        Intent intent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);
        intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "zh-TW");
        intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM);
        intent.putExtra(RecognizerIntent.EXTRA_PARTIAL_RESULTS, true);
        speechRecognizer.startListening(intent);

        btn.setText(R.string.stt_stop);
    }

    private void stopSTT(){
        speechRecognizer.stopListening();
        stopUIUpdate();
    }

    private void stopUIUpdate(){
        isInProgress.set(false);
        btn.setText(R.string.stt_start);
    }

    class SpeechRecognizerListener implements RecognitionListener {
        private static final String TAG = "RecognitionListener";

        @Override
        public void onReadyForSpeech(Bundle bundle) {
            Log.d(TAG, "onReadyForSpeech");
        }

        @Override
        public void onBeginningOfSpeech() {
            Log.d(TAG, "onBeginningOfSpeech");
        }

        @Override
        public void onRmsChanged(float v) {
            Log.d(TAG, "onRmsChanged: Rms changed to " + v + " dB");
        }

        @Override
        public void onBufferReceived(byte[] bytes) {
            Log.d(TAG, "onBufferReceived");
        }

        @Override
        public void onEndOfSpeech() {
            Log.d(TAG, "onEndOfSpeech");
        }

        @Override
        public void onError(int i) {
            Log.e(TAG, "onError: " + i);
            switch (i) {
                case SpeechRecognizer.ERROR_AUDIO:
                    textView.setText(R.string.audio_recording_error);
                    break;
                case SpeechRecognizer.ERROR_CLIENT:
                    textView.setText(R.string.client_side_error);
                    break;
                case SpeechRecognizer.ERROR_INSUFFICIENT_PERMISSIONS:
                    textView.setText(R.string.insufficient_permissions);
                    break;
                case SpeechRecognizer.ERROR_NETWORK:
                    textView.setText(R.string.network_error);
                    break;
                case SpeechRecognizer.ERROR_NETWORK_TIMEOUT:
                    textView.setText(R.string.network_timeout);
                    break;
                case SpeechRecognizer.ERROR_NO_MATCH:
                    textView.setText(R.string.no_match);
                    break;
                case SpeechRecognizer.ERROR_RECOGNIZER_BUSY:
                    textView.setText(R.string.recognition_service_busy);
                    break;
                case SpeechRecognizer.ERROR_SERVER:
                    textView.setText(R.string.error_from_server);
                case SpeechRecognizer.ERROR_SPEECH_TIMEOUT:
                    textView.setText(R.string.no_speech_input);
                    break;
                default:
                    textView.setText(R.string.default_speech_input);
            }
            stopUIUpdate();
        }

        @Override
        public void onResults(Bundle bundle) {
            Log.d(TAG, "onResults");
            stopUIUpdate();
            StringBuilder text = new StringBuilder();
            ArrayList<String> results = bundle.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION);
            if (results != null) {
//                for (String r : results) {
//                    text.append(r);
//                }
                text.append(results.get(0));
            } else {
                text.append("Empty result");
            }

            textView.setText(text);
        }

        @Override
        public void onPartialResults(Bundle bundle) {
            Log.d(TAG, "onResults");
            StringBuilder text = new StringBuilder();
            ArrayList<String> results = bundle.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION);
            if (results != null) {
                for (String r : results) {
                    text.append(r);
                }
            } else {
                text.append("Empty result");
            }

            textView.setText(text);
        }

        @Override
        public void onEvent(int i, Bundle bundle) {
            Log.d(TAG, "onEvent: " + i);
        }
    }
}